/*
Macro Name	: csbmva_Chsqp
Function	: Computes Chi Square statistic and p-value for the chi-square statistics .
Created by	: BIS Team
Date		: sep 07, 2010
SAS Version	: 9.2


Parameters	: 

input_ds		: Name of the input data set 
output_ds		: Name of the data set which stores brier score.
flag :The flag denotes that it is used for storing measure value	
Input Dataset(s) Format:
For LGD
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
actual_lgd	 				Number		Default Formatting
ESTIMATED_LGD            Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------


For CCF
--------------------------------------------------------------------------
Name				Type		Format
--------------------------------------------------------------------------
Pool_Name			Character 	Default Formatting	
Pool_Seq_no	 		Number		Default Formatting
Pool_Sk	 		Number		Default Formatting
score_time_sk           Number		Default Formatting
level_sk                Number		Default Formatting
Model_sk                 Number		Default Formatting
Model_rk                 Number		Default Formatting
No_Of_records_Actual	Number		Default Formatting
No_of_records_Dev	 	Number 		Default Formatting
actual_ccf	 				Number		Default Formatting
ESTIMATED_ccf            Number		Default Formatting
No_Of_Actual_Bads	Number		Default Formatting	
--------------------------------------------------------------------------


Output Dataset(s) Format:
output_ds
-------------------------------------------------------------------------
Name			Type		Format
------------------------------------------------------------------------
stat			Numeric		Default Format
chsqp	Numeric		Default Format
------------------------------------------------------------------------

Logic:

The chi-square goodness of fit statistic is computed as follows.
1. Chi sq = Sum((O_i - E_i)^2 /E_i)
2. Under Null hypothesis this follows Chi Square distribution  with K-1 df. where K No of classes/pools
3. p_value is calculated as P_0[Chi_SQ > computed_Chi Sq] = 1 - P_0[Chi_SQ <= computed_Chi Sq]

Notes:
1. It is assumed that data set of specified as input_ds is present and has necessarry required rights.
2. It is assumed that data set of specified as input_ds has valid specified fileds.
3. For all output datasets (Viz output_ds),it is assumend that
	a. it has valid name.
	b. Macro has necessary required rights to create the datasets with specified name @ specified location.
	c. Name of the data-set is unique OR if dataset with similar name already exists then it can be safely overwritten.

*/
options mlogic mprint;
%macro csbmva_chsqp(input_ds=,output_ds=,flag=,model_type=);


%if &flag = 1 %then %do;
data _NULL_;

name=put(time(),mmss8.1);

name1=kscan(name,1,":.");

name2=kscan(name,2,":.");

name3=kscan(name,3,":.");

name4=ktrim(kleft(name1))||ktrim(kleft(name2))||ktrim(kleft(name3));

call symput('log_name',"&LOG_PATH_CS"||"&sysmacroname"||ktrim(kleft(name4))||".log");

run;


%end;


%if &model_type = &LGD_MODEL %then %do;


proc sort data=&input_ds out=work.sorted_feed;
by pool_seq_no;
quit;

data Chisq;
	retain CHI_SQ .;
	set work.sorted_feed end = last ;
	if estimated_lgd ne 0 then
	CHI_SQ = sum(CHI_SQ,(estimated_lgd-actual_lgd)*(estimated_lgd-actual_lgd)/estimated_lgd);
run;
%let err_code =;
proc sql noprint;
	select count(pool_name)-1 into:DF from chisq;
	create table chisq1 as
	select (CHI_SQ) as stat from chisq having pool_seq_no=max(pool_seq_no);
	select (CHI_SQ) into:chi from chisq having pool_seq_no=max(pool_seq_no);
quit;

%if %sysevalf(&chi>=0)  %then %do;
data &output_ds;
	set chisq1;
	chsqp= 1 - CDF('CHISQUARE', stat, &DF); 
run;
%end;

%else  %goto Error_Handler1;

%csbmva_errorhandler (err_code);

%if (&err_code ne 'NONE')%then %goto Error_Handler1;

%return;


%Error_Handler1:
%put WARNING: &err_code;
%csbmva_defdata(input_ds=,stat='CHISQ',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);
		
%end;


%if &model_type = &CCF_MODEL %then %do;

proc sort data=&input_ds out=work.sorted_feed;
by pool_seq_no;
quit;

data Chisq;
	retain CHI_SQ .;
	set work.sorted_feed end = last ;
	if estimated_CCF ne 0 then
	CHI_SQ = sum(CHI_SQ,(estimated_CCF-actual_CCF)*(estimated_CCF-actual_CCF)/estimated_CCF);
run;
%let err_code =;
proc sql noprint;
	select count(pool_name)-1 into:DF from chisq;
	create table chisq1 as
	select (CHI_SQ) as stat from chisq having pool_seq_no=max(pool_seq_no);
	select (CHI_SQ) into:chi from chisq having pool_seq_no=max(pool_seq_no);
quit;
%if %sysevalf(&chi>=0)  %then %do;
data &output_ds;
	set chisq1;
	chsqp= 1 - CDF('CHISQUARE', stat, &DF); 
	
run;
%end;

%else  %goto Error_Handler2;
/*****************************************************************/
%csbmva_errorhandler (err_code);

%if (&err_code ne 'NONE')%then %goto Error_Handler2;


%return;
/*****************************************************************/
%Error_Handler2:
%put WARNING: &err_code;
%csbmva_defdata(input_ds=,stat='CHISQ',
		out_graph_ds =,output_ds =&output_ds ,out_stat_ds=);
		

%end;

%mend csbmva_chsqp;
