/**********************************************************************************************
Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.

Module         : csbmva_model_dev_fact.sas
Function       : This macro creates model_dev_fact.

Authors        :  CSB Team
Date           :  17JAN2011
SAS            :  9.2
Called-by      :  MODEL_DEV_FACT job
Calls          :  %BANKFDN_RCSET macro to capture the return code for error handling.
				  %CSBMVA_MDL_DEV_FACT_DATA_PREP to get other scores

Input Datasets :  CSB_MDL_INPUT (MDLMART library)
                  MDL_MODEL_DETAILS (CSBSCR library)
				 

Output Datasets:  _<MODEL_RK>_DEV_FACT (CSB Fact library)

Logic          :  1. Extract rows from CSB_MDL_INPUT table which have its status as 
					 'Running'. Fact will be created in the order of PD, LGD, CCF.
				  2. For each model following steps are followed
					 i)   Get model_nm , odds table name, model level, pool scheme type
						  model type.
					 ii)  In odds table name, rename the level column to RECORD_RK.
					 iii) Depending upon the type of model rename TGT_VAR to ACTUAL_GGO_BAD
						  or ACTUAL_LGD or ACTUAL_CCF.
					 iv)  If model is LGD model, the call %csbmva_get_pd_values macro to populate
                          the estimated_pd & score_points column.
					 v)   If If model is CCF model, the call %csbmva_get_pd_values & 
                          %csbmva_get_lgd_values macro to bring pd & lgd values.
                     vi)  Join it with MLD_POOL_DIM_EXT to assign pools to individual
                          Account/Customer.
					 
Parameters     :  None
**********************************************************************************************/

%macro csbmva_model_dev_fact;

%local trans_rc;

proc sql;
select count(*) into: model_cnt
from csbscr.mdl_model_details;
quit;

%let model_cnt = &model_cnt;
%bankfdn_rcset(&sqlrc);

%bankfdn_rcset(&syserr);
*==================================================;
* Looping through individual models ;
*==================================================;

%do i = 1 %to %eval(&model_cnt);

	%let trans_rc = 0;
	%local m_model_rk m_level_rk m_odds_table m_model_type level_var output_ds m_model_id  m_scr_abt_lib scr_ds_nm;

	data _null_;
		pointer = &i;
		set csbscr.mdl_model_details point = pointer;
		call symputx('m_model_rk',model_rk);
		call symputx('m_level_sk',level_sk);
		call symputx('m_odds_table',scored_abt_table_nm);
        call symputx('m_model_type',model_type_cd);
		call symputx('m_model_id',model_id);
		call symputx('m_scr_abt_lib',scored_abt_libref);
		stop;
	run;

%if (&m_odds_table ne %str()) %then %do;


  %let m_odds_table = &m_odds_table;
  %let m_scr_abt_lib = &m_scr_abt_lib;
  %let m_model_rk = &m_model_rk;
  %let m_model_type = &m_model_type;
  %let m_model_id = &m_model_id;
  %let m_level_sk=&m_level_sk;

  %let m_model_type = %str(%')&m_model_type%str(%');

  %if (&m_scr_abt_lib eq %str()) %then %do;
     %let m_scr_abt_lib = cs_mdl;
  %end;

*==================================================;
* Getting m_mdl_src_type_cd ;
*==================================================;

proc sql noprint;
  select model_source_type_cd into :m_mdl_src_type_cd
  from &dabt_lib..model_master a,&dabt_lib..model_source_type_master b
  where a.model_sk = &m_model_rk
  and a.model_source_type_sk=b.model_source_type_sk;
quit;
%bankfdn_rcset(&sqlrc);
*==================================================;
*Renaming Logic ;
*==================================================;

proc sql noprint;
  select VARIABLE_COLUMN_NM into :m_outcome_var
  from &dabt_lib..model_x_act_outcome_var as a,
       &dabt_lib..variable_master as b
  where a.variable_sk  = b.variable_sk
  and   a.target_variable_flg = 'Y'
  and   a.model_sk = &m_model_rk;
quit;
%bankfdn_rcset(&sqlrc);

%let m_outcome_var =&m_outcome_var;
%let scr_ds_nm=&m_odds_table.;

data &scr_ds_nm._temp;
set &m_scr_abt_lib..&m_odds_table;
%if %kupcase(&m_model_type) = &PD_ONG_MODEL. or %upcase(&m_model_type) = &PD_APP_MODEL. %then %do;
	  rename &m_outcome_var = tgt_var;
	%end;
%else %if %kupcase(&m_model_type) = &LGD_MODEL. %then %do;
      rename LGD = tgt_var;
	%end;
%else %if %kupcase(&m_model_type) = &CCF_MODEL. %then %do;
	  rename &m_outcome_var = tgt_var;
	%end;
run;
%bankfdn_rcset(&syserr);

data &scr_ds_nm;
set &scr_ds_nm._temp;
    %if %kupcase(&m_model_type) = &PD_ONG_MODEL. or %kupcase(&m_model_type) = &PD_APP_MODEL. %then %do;
       	 	%if &m_mdl_src_type_cd ne JDG_SCR %then %do;
	 	 	rename P_&m_outcome_var.1 = ESTIMATED_PD;
		%end;
		%else %do ;
			ESTIMATED_PD=.;
		%end;
        %end;
	%else %if %kupcase(&m_model_type) = &LGD_MODEL. %then %do;
      rename P_LGD = ESTIMATED_LGD;
	%end;
	%else %if %kupcase(&m_model_type) = &CCF_MODEL. %then %do;
	  rename P_&m_outcome_var = ESTIMATED_CCF;
	%end;
  run;
quit;

%bankfdn_rcset(&syserr);


%if %kupcase(&m_model_type) = &PD_ONG_MODEL. or %kupcase(&m_model_type) = &PD_APP_MODEL. %then %do;
  data &scr_ds_nm;
    set &scr_ds_nm;
    scorecard_points = round(scorecard_points);
	if tgt_var not in ( 1 . ) then tgt_var = 0;
  run;
%end;

%if %sysfunc(exist(csbscr.&scr_ds_nm)) %then %do;
	proc sql;
		drop table csbscr.&scr_ds_nm;
	quit;
%end;

data csbscr.&scr_ds_nm;
  set &scr_ds_nm;
run;

*--------------------------------------------------;
* Getting level of each associated model ;
*--------------------------------------------------;
	proc sql ;
		select kupcase(LEVEL_KEY_COLUMN_NM) into :level_var
		from  &csb_apdm_libref..level_master as a
		where a.level_sk = %eval(&m_level_sk);	
	quit;
        %let level_var=&level_var;

	%bankfdn_rcset(&sqlrc);
*--------------------------------------------------;
* CREATE OUTPUT TABLE ;
*--------------------------------------------------;
%let output_ds=work.dev_fact_prep_ds;

%if &m_model_type=&PD_ONG_MODEL or &m_model_type=&PD_APP_MODEL %then %do;

	proc sql;
	  create table &output_ds as
	     select &level_var, 
				case 
					when tgt_var not in ( 1 . ) then 0
					else tgt_var
				 end as ACTUAL_GOOD_BAD, estimated_pd,
		 		. as ACTUAL_LGD, . as ESTIMATED_LGD,
				. as ACTUAL_CCF, . as ESTIMATED_CCF,
				scorecard_points as SCORE_POINTS
	     from &m_odds_table;
	quit;
	%bankfdn_rcset(&sqlrc);


%end;


%if &m_model_type=&LGD_MODEL %then %do;

	proc sql;
	  create table &output_ds as
	     select &level_var, tgt_var as ACTUAL_LGD, ESTIMATED_LGD,
		 		. as ACTUAL_GOOD_BAD, . as ESTIMATED_PD,
				. as ACTUAL_CCF, . as ESTIMATED_CCF,
				. as SCORE_POINTS
	     from &m_odds_table;
	quit;
	%bankfdn_rcset(&sqlrc);
%end;

%if &m_model_type=&CCF_MODEL %then %do;

	proc sql;
	  create table &output_ds as
	     select &level_var, tgt_var as ACTUAL_CCF , ESTIMATED_CCF ,
		 		. as ACTUAL_GOOD_BAD, . as ESTIMATED_PD,
				. as ACTUAL_LGD, . as ESTIMATED_LGD,
				. as SCORE_POINTS
	     from &m_odds_table;
	quit;
	%bankfdn_rcset(&sqlrc);

%end;

	
*--------------------------------------------------;
* Creating MODEL DEV FACT ;
*--------------------------------------------------;
		
	proc sql ;
	    create table &cs_fact_lib.._&m_model_rk._dev_fact as
		  select &level_var as	RECORD_RK,
		         SCORE_POINTS, ESTIMATED_PD, ESTIMATED_LGD, ESTIMATED_CCF,
			     ACTUAL_GOOD_BAD, ACTUAL_LGD, ACTUAL_CCF,
				 &m_model_rk as MODEL_RK, &m_level_sk as LEVEL_SK
		  from work.dev_fact_prep_ds;
	quit;

	%if &trans_rc > 4 %then %do;
    	proc sql noprint;
      		update mdlmart.csb_mdl_input
      		set status = &ERROR_STATUS
      		where model_id = "&m_model_id";
    	quit;
		%bankfdn_rcset(&sqlrc);
  	%end;
  	%else %do;
		proc datasets lib = &cs_fact_lib. nodetails;
	  		modify _&m_model_rk._dev_fact;
			index create record_rk / nomiss;
        		attrib _all_ label = '';
		quit;
  	%end;


%end;
%else %do;

	proc sql ;
  CREATE TABLE &cs_fact_lib.._&m_model_rk._dev_fact (        SCORE_POINTS         NUMERIC(8) ,        ESTIMATED_PD         NUMERIC(8) ,
        RECORD_RK            NUMERIC(8) FORMAT=12. INFORMAT=12.,        ESTIMATED_LGD        NUMERIC(8) ,        ESTIMATED_CCF        NUMERIC(8) ,
        ACTUAL_GOOD_BAD      NUMERIC(8) ,        ACTUAL_LGD           NUMERIC(8) ,        ACTUAL_CCF           NUMERIC(8) ,
        MODEL_RK             NUMERIC(8) FORMAT=12. INFORMAT=12.,
        LEVEL_SK             NUMERIC(10)  );
 CREATE INDEX RECORD_RK ON &cs_fact_lib.._&m_model_rk._dev_fact  (RECORD_RK );

quit;

%end;
%end;

%mend csbmva_model_dev_fact;
