/************************************************************************
 * Copyright (c) 2010 by SAS Institute Inc., Cary, NC, USA.            	*
 *                                                                     	*
 * NAME:          csbmva_pool_lgd_measure_fact_dev                     	*
 *                                                                     	*
 * PURPOSE:       Called by csbmva_pool_dev_data_load to calculate      *
 *				  developement statistics for LGD models*
 *                                                                     	*
 * USAGE:         %csbmva_pool_lgd_measure_fact_dev			*
 *			(model_key= ,					*
 *			time_key = ,					*
 *				measure_fact_lib = ,			*
 *				measure_fact_ds = , 			*
 *			model_type = ,					*
 *			pool_mapped_fact_ds=,pool_scheme_sk=)	        *
 *                                                                     	*
 * PARAMETERS:    model_key   	 The model identifier as present in     *
 *					apdm.pool_scheme          	*
 *		  time_key       The surrogate key for  score time 	*
 *				date in POOL_SCHEME_DEFN     	 	*
 *		  measure_fact_lib   The library which contains the     *
 *		   			measure fact:  		        *
 *		  measure_fact_ds    The name of fact dataset which     *
 *				     contains the measure values 	*
 *		  model_type         The type of model 			*
 *		  pool_mapped_fact_ds    The name of fact dataset which *
 *					 contains the pool information  * 
 *				     mapped to record level pooling abt *
 *                pool_scheme_sk   Pool scheme number                   *
 ***********************************************************************/
%macro csbmva_pool_lgd_measure_fact_dev(model_key        = ,time_key        = ,
                                   measure_fact_lib = ,measure_fact_ds = ,
                                   model_type = ,pool_mapped_fact_ds=,pool_scheme_sk=);

*==================================================;
* Defining local macro variables ;
*==================================================;

%local  m_level_sk ;

*==================================================;
* Finding level  of model ;
*==================================================;

proc sql noprint;
	select 
		level_sk 
	into 
		:m_level_sk
  	from 
		&dabt_lib..model_master_extension 
  	where 
		model_key = &model_key ;
quit;

%bankfdn_rcset(&sqlrc);

*==================================================;
* Grouping the data at pool level ;
*==================================================;
%let tot_obs_in_pool_fact=0;
%let dsid=%sysfunc(open(&pool_mapped_fact_ds));
%let tot_obs_in_pool_fact=(%eval(%sysfunc(attrn(&dsid,nobs))));
%let rc=%sysfunc(close(&dsid));
%if %eval(&tot_obs_in_pool_fact) gt 0 %then %do;
*==================================================================;
* Read variable mapping for the pool_scheme ;
*==================================================================;
Proc sql;
	select INPUTVARCOUNT,INPUTVAR1,INPUTVAR2,INPUTVAR3,INPUTVAR4
			into :varcount,:invar1, :invar2, :invar3, :invar4
	from &csb_apdm_libref..Pool_scheme_variable_map
	where pool_scheme_sk=%eval(&pool_scheme_sk);
quit;
%bankfdn_rcset(&sqlrc);
%let invar1=&invar1;
%let invar2=&invar2;
%let invar3=&invar3;
%let invar4=&invar4;
*==================================================================;
* Extract pool_dim data for open ended bins  ;
*==================================================================;

Proc sql;
	create table pool_dim_ext as
	select *,coalesce(pd_from_no, -9999999) as  pd_from_no_oped,coalesce(pd_to_no, 9999999) as  pd_to_no_oped,
		coalesce(lgd_from_no, -9999999) as  lgd_from_no_oped,coalesce(lgd_to_no, 999999) as  lgd_to_no_oped,
		coalesce(ccf_from_no, -9999999) as  ccf_from_no_oped,coalesce(ccf_to_no, 9999999) as  ccf_to_no_oped,
		coalesce(score_from_no, -9999999) as  score_from_no_oped,coalesce(score_to_no, 9999999) as  score_to_no_oped
	from &csb_apdm_libref..pool_dim
	where pool_scheme_sk=&pool_scheme_sk
		and valid_end_dttm = &valid_high_dttm.;
quit;
%bankfdn_rcset(&sqlrc);
proc sql;
	create table &debug_lib.._&model_key._model_dev_pool_lgd_temp as
    select 
		b.pool_sk,
		b.pool_name,
		b.pool_seq_no, 
		case 
			when sum(a.record_rk) = 0 then .
			else count(a.record_rk) 
		end as no_of_records_actual, 
	  	avg(a.estimated_lgd) as estimated_lgd,
	  	avg(a.actual_lgd) as actual_lgd,
	  	sum(a.actual_good_bad) as no_of_actual_bads
	from 
		&pool_mapped_fact_ds a right outer join pool_dim_ext b
	on b.pool_scheme_sk=&pool_scheme_sk
			%do m=1 %to &varcount;
				and &&invar&m.._from_no_oped < 
                                     %if %kupcase(&&invar&m) ne SCORE %then %do; estimated_&&invar&m  %end;
				       %else %if %kupcase(&&invar&m) eq  SCORE %then %do; score_points  %end;
				<= &&invar&m.._to_no_oped				
       		%end; 
	where b.pool_scheme_sk=&pool_scheme_sk 
	group by 
		pool_sk,
		pool_name,
		pool_seq_no;
 	
quit;
%bankfdn_rcset(&sqlrc);
%end;
%else %do;
	proc sql;
  		create table &debug_lib.._&model_key._model_dev_pool_lgd_temp as
	select 
		a.pool_sk,
		a.pool_name,
		a.pool_seq_no,		
		a.no_of_records as no_of_records_actual, 
		b.rate as estimated_lgd,
                . as actual_lgd,
		. as no_of_actual_bads
	from 
		 &csb_apdm_libref..pool_dim a 
			left outer join
		 &csb_apdm_libref..model_x_pool b
		 on         a.pool_sk=b.pool_sk and
                    b.model_rk=&model_key and
                    a.valid_end_dttm=&valid_high_dttm
		where a.pool_scheme_sk=&pool_scheme_sk
	;  
quit;
%bankfdn_rcset(&sqlrc);	
%end;
*==================================================;
* Creating pool level feed;
*==================================================;

proc sql;
  	create table &debug_lib.._&model_key._&time_key._LGD_POOL_FD as
 	select 
		pool_sk,
		pool_name, 
		pool_seq_no,
	    &EPSILON as no_of_records_dev, 
	    coalesce(no_of_records_actual, &EPSILON) as no_of_records_actual, 
	    no_of_actual_bads,
        estimated_lgd,
		actual_lgd,
		&model_key as model_rk,
		&m_level_sk as level_sk, 
		&time_key as score_time_sk
	from 
		&debug_lib.._&model_key._model_dev_pool_lgd_temp
	order by 3;
quit;
%bankfdn_rcset(&sqlrc);

*==================================================;
* Populating model_measure_fact;
*==================================================;

%csbmva_cal_measures_lgd(model_key      = &model_key      ,time_key       = &time_key,
                         model_type     = &model_type     ,feed_lib       = &debug_lib,
					     feed_ds_pool   = _&model_key._&time_key._LGD_POOL_FD,
						 feed_ds_record = ,
					     measure_ds_lib = &debug_lib      ,measure_ds_nm  = measure_ds_&model_key.,
						 isDevelopmentCycle = true,
						 isPoolBased = true);

%csbmva_insert_measure_val(model_key        = &model_key       ,time_key        = &time_key,
                           measure_fact_lib = &measure_fact_lib,measure_fact_ds = &measure_fact_ds,
                           measure_ds_lib   = &debug_lib       ,measure_ds_nm   = measure_ds_&model_key.,
                           dev_cnt          = 1 ,isDevelopmentCycle = true );

%csbmva_measure_traffic(model_key        = &model_key       ,time_key        = &time_key,
                        measure_fact_lib = &measure_fact_lib,measure_fact_ds = &measure_fact_ds,
                        model_type       = &model_type      ,dev_cnt         = 1);

%csbmva_measure_level_traffic(model_key        = &model_key       ,time_key        = &time_key,
                              measure_fact_lib = &measure_fact_lib,measure_fact_ds = &measure_fact_ds,
                              model_type       = &model_type      ,dev_cnt         = 1);

%mend csbmva_pool_lgd_measure_fact_dev;
