
	/* Post action macro called after actual job calculation is done and ARM is populated*/
%macro csbmva_post_actual_calc_wrapper(m_action_cd=, m_purpose_sk=, m_key_sk=);

	%local m_purpose_cd;
	
	%let m_purpose_cd = ;
	proc sql noprint;
		select kupcase(pm.purpose_cd) into :m_purpose_cd
		from &dabt_lib..purpose_master pm
		where pm.purpose_sk = &m_purpose_sk;
	quit;

    %let m_purpose_cd = &m_purpose_cd;

	/* If update LGD post build flag is set to Y then update the LGD. 
		This parameter value is taken from apdm.parameter_value_dtl table.*/
	%if &m_purpose_cd eq LGD and %upcase(&CSB_UPDT_LGD_POST_BUILD_FLG) eq Y %then %do; /*i18NOK:LINE */
		%let m_abt_type_cd = ACT;

		%let m_level_cd = ;
		%let m_level_sk = ;
		%let m_model_sk = ;
		%let m_scr_model_sk = ;
		%let m_scr_control_dtl_sk = ;
		%let m_tbl_nm = ;
		proc sql noprint;
			select kupcase(lm.level_cd), lm.level_sk, sm.model_sk, sm.scoring_model_sk, arcd.scoring_control_detail_sk, arcd.actual_result_abt_nm 
				 into :m_level_cd, :m_level_sk, :m_model_sk, :m_scr_model_sk, :m_scr_control_dtl_sk, :m_tbl_nm
			from &dabt_lib..actual_result_control_detail arcd
		  		inner join &dabt_lib..scoring_control_detail scd
				on (arcd.scoring_control_detail_sk = scd.scoring_control_detail_sk)
				inner join &dabt_lib..scoring_model sm
				on (scd.scoring_template_sk = sm.scoring_template_sk)
		  		inner join &dabt_lib..model_master mm
				on (sm.model_sk = mm.model_sk)
				inner join &dabt_lib..level_master lm
				on (mm.level_sk = lm.level_sk)
			where arcd.actual_result_control_dtl_sk = &m_key_sk;
		quit;

	   	%let m_level_cd = &m_level_cd;
		%let m_level_sk = &m_level_sk;
		%let m_model_sk = &m_model_sk;
		%let m_scr_model_sk = &m_scr_model_sk;
		%let m_scr_control_dtl_sk = &m_scr_control_dtl_sk;
		%let m_tbl_nm = &m_tbl_nm;

		%let m_scoring_dttm = ;
		data _null_;
			set &dabt_lib..scoring_control_detail;
			where scoring_control_detail_sk = &m_scr_control_dtl_sk;
			call symput('m_scoring_dttm',cat("'",strip(put(scoring_as_of_dttm,datetime.)),"'DT")); /*i18NOK:LINE */
		run;
		%let m_scoring_dttm = &m_scoring_dttm;

		%let m_sub_of_analysis_cd = &m_level_cd;
		%let m_sub_rk_col_nm = ;
		
		/* Get key column name */
		/* Assumption: key constitutes of only one column */
		proc sql noprint;
			select source_column_nm into :m_sub_rk_col_nm
			from &dabt_lib..level_key_column_dtl lkcd inner join
				&dabt_lib..source_column_master scm
				on (lkcd.key_column_sk = scm.source_column_sk)
			where upcase(key_type_cd) = "RETAIN_KEY" and lkcd.level_sk = &m_level_sk; /*i18NOK:LINE */
		quit;
		%let m_sub_rk_col_nm = &m_sub_rk_col_nm;

		
		%let m_tbl_libref = &DABT_ACTUAL_CALC_ABT_LIBREF;
		%let m_lgd_column_nm = ACT_LGD;
		%let m_ead_column_nm = ACT_EAD;
		%let m_def_dt_column_nm = DEFAULT_DT;

		
		%let m_scr_tbl_libref = ARM;
		%let m_scr_tbl_nm = ;

		/* Get table name containing the population */
		proc sql noprint;
			select ara_scoring_table_nm into :m_scr_tbl_nm
			from &dabt_lib..purpose_level_config_dtl plcd
			where plcd.purpose_sk = &m_purpose_sk and plcd.level_sk = &m_level_sk;
		quit;
		%let m_scr_tbl_nm = &m_scr_tbl_nm;

		%let m_sub_retain_key_lib = dbscr;
		%let m_sub_retain_key_tbl_nm = act_calc_subset_&m_key_sk.;

		%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
    		option &DABT_INDB_OPT_PRE_BLD_ABT_JOB;
		%end;
		
		%dabt_drop_table(m_table_nm=&m_sub_retain_key_lib..&m_sub_retain_key_tbl_nm);
		
		proc sql noprint;
			create table &m_sub_retain_key_lib..&m_sub_retain_key_tbl_nm as
			select * 
			from &m_scr_tbl_libref..&m_scr_tbl_nm
			where scoring_model_sk = &m_scr_model_sk and scoring_as_of_dttm = &m_scoring_dttm;
		quit;

		%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
		    options NODBIDIRECTEXEC;
		%end;

		%csbmva_calc_upd_lgd_var_wrapper(m_abt_type_cd=&m_abt_type_cd, m_object_sk=&m_key_sk, m_subject_of_analysis_cd=&m_sub_of_analysis_cd,
									m_subject_retain_key_libref=&m_sub_retain_key_lib,m_subject_retain_key_tbl_nm=&m_sub_retain_key_tbl_nm,
									m_subject_rk_column_nm=&m_sub_rk_col_nm, m_to_be_upd_tbl_libref=&m_tbl_libref,
									m_to_be_upd_tbl_nm=&m_tbl_nm, m_to_be_upd_lgd_column_nm=&m_lgd_column_nm,
									m_to_be_upd_ead_column_nm=&m_ead_column_nm,m_to_be_upd_def_dt_column_nm=&m_def_dt_column_nm);
	%end;

%mend csbmva_post_actual_calc_wrapper;
