%macro csbmva_setdefaults(m_action_cd = ,m_purpose_sk = ,m_key_sk = );

%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
    option &DABT_INDB_OPT_PRE_BLD_ABT_JOB;
	libname cs_mdl clear;
	libname cs_mdl teradata database=&csb_mdl_abt_schema. server="&fm_server." authdomain=&fm_authdomain.;
%end;


%let m_action_cd = &m_action_cd;
%let m_purpose_sk = &m_purpose_sk;
%let m_abt_sk = &m_key_sk;
%let m_src_lib = &m_src_lib;

proc sql noprint;
  select project_sk,abt_table_nm,last_built_stacked_type_flg into :m_project_sk,:m_abt_table_nm,:m_abt_stk_flg
  from &m_src_lib..modeling_abt_master
  where abt_sk = &m_abt_sk;
quit;

%let m_project_sk = &m_project_sk;
%let m_abt_table_nm = &m_abt_table_nm;
%let m_abt_stk_flg = &m_abt_stk_flg;

%local m_purpose_cd;

%let m_purpose_cd = ;
proc sql noprint;
	select kupcase(pm.purpose_cd) into :m_purpose_cd
	from &dabt_lib..purpose_master pm
	where pm.purpose_sk = &m_purpose_sk;
quit;

%let m_purpose_cd = &m_purpose_cd;

/* If update LGD post build flag is set to N then effect of hotfix won't be coming in LGD process. It typically the case if some implementataion has already customized and implemented LGD and want to continue with it. This parameter value is taken from apdm.parameter_value_dtl table.*/
%if &m_purpose_cd eq LGD and %upcase(&CSB_UPDT_LGD_POST_BUILD_FLG) eq N %then %do; /*i18NOK:LINE */
	%let dsid = %sysfunc(open(&DABT_MODELING_ABT_LIBREF..&m_abt_table_nm));
	%let vnum = %sysfunc(varnum(&dsid,build_dttm));
	%let clsid = %sysfunc(close(&dsid));

	%if &vnum = 0 %then %do;
	  %let m_abt_stk_flg = N;
	%end;
	%else %do;
	  %let m_abt_stk_flg = Y;
	%end;

	%if &m_abt_stk_flg eq N %then %do; /* ABT built for single time period */

	  proc sql noprint;
	    select datepart(last_built_as_of_dttm) into :m_abt_bld_dt
	    from &m_src_lib..mdl_abt_last_run_as_of_dt_dtl
	    where abt_sk = &m_abt_sk;
	  quit;

	%if &FM_GRAIN. eq 1 %then %do; /* For Foundation Mart data grain = Monthly */

	  proc sql noprint;
	    update &DABT_MODELING_ABT_LIBREF..&m_abt_table_nm.
	    set input_default_dt = .
	    where (input_default_dt < intnx('month',&m_abt_bld_dt,-&DEFAULT_CAPTURE_PERIOD.+1,'begin') or default_status_cd not in (&DEFAULT_STATUS_CD.)) 
		  and input_default_dt is not null;
	  quit;

	%end;
	%else %if &FM_GRAIN. eq 3 %then %do; /* For Foundation Mart data grain = Daily */

	  proc sql noprint;
	    update &DABT_MODELING_ABT_LIBREF..&m_abt_table_nm.
	    set input_default_dt = .
	    where (input_default_dt < intnx('day',&m_abt_bld_dt,-&DEFAULT_CAPTURE_PERIOD.+1,'begin') or default_status_cd not in (&DEFAULT_STATUS_CD.))
		  and input_default_dt is not null;
	  quit;

	%end;

	%end;

	%else %if &m_abt_stk_flg eq Y %then %do; /* ABT built for multiple time period */

	%if &FM_GRAIN. eq 1 %then %do; /* For Foundation Mart data grain = Monthly */

	  proc sql noprint;
	    update &DABT_MODELING_ABT_LIBREF..&m_abt_table_nm.
	    set input_default_dt = .
	    where (%bankfdn_indb_dhms(input_default_dt) < %bankfdn_indb_intnx('dtmonth',build_dttm,-&DEFAULT_CAPTURE_PERIOD.+1,'begin') or default_status_cd not in (&DEFAULT_STATUS_CD.))
		  and input_default_dt is not null;
	  quit;

	%end;
	%else %if &FM_GRAIN. eq 3 %then %do; /* For Foundation Mart data grain = Daily */

	  proc sql noprint;
	    update &DABT_MODELING_ABT_LIBREF..&m_abt_table_nm.
	    set input_default_dt = .
	    where (%bankfdn_indb_dhms(input_default_dt) < %bankfdn_indb_intnx('dtday',build_dttm,-&DEFAULT_CAPTURE_PERIOD.+1,'begin') or default_status_cd not in (&DEFAULT_STATUS_CD.))
		  and input_default_dt is not null;
	  quit;

	%end;

	%end; 

	%if &fm_db_eng.=TERADATA %then %do;
		PROC SQL;
		CONNECT TO TERADATA (server="&fm_server." authdomain=&fm_authdomain. mode=TERADATA) ;
		EXECUTE (alter table &csb_mdl_abt_schema..&m_abt_table_nm. drop ACT_EAD;
			) BY TERADATA ;
			DISCONNECT FROM TERADATA ;
		quit;

		PROC SQL;
		CONNECT TO TERADATA (server="&fm_server." authdomain=&fm_authdomain. mode=TERADATA) ;
		EXECUTE (alter table &csb_mdl_abt_schema..&m_abt_table_nm. drop ACT_VALUE_AT_RECOVERY;
			) BY TERADATA ;
			DISCONNECT FROM TERADATA ;
		quit;

			PROC SQL;
		CONNECT TO TERADATA (server="&fm_server." authdomain=&fm_authdomain. mode=TERADATA) ;
		EXECUTE (alter table &csb_mdl_abt_schema..&m_abt_table_nm. drop ACT_RECOVERY_COST;
			) BY TERADATA ;
			DISCONNECT FROM TERADATA ;
		quit;

			PROC SQL;
		CONNECT TO TERADATA (server="&fm_server." authdomain=&fm_authdomain. mode=TERADATA) ;
		EXECUTE (alter table &csb_mdl_abt_schema..&m_abt_table_nm. drop ACT_LGD;
			) BY TERADATA ;
			DISCONNECT FROM TERADATA ;
		quit;

	%end;


	%else %if &fm_db_eng.=SAS %then %do; /* Start for SAS */
	proc sql;
	  alter table &DABT_MODELING_ABT_LIBREF..&m_abt_table_nm.
	  drop ACT_EAD, ACT_VALUE_AT_RECOVERY, ACT_RECOVERY_COST, ACT_LGD;
	quit;
	%end;


	%if &DABT_INDB_PROCESSING_FLG eq Y %then %do;
	    options NODBIDIRECTEXEC;
		libname cs_mdl clear;
		libname cs_mdl teradata database=&csb_mdl_abt_schema. server="&fm_server." authdomain=&fm_authdomain. mode=TERADATA;
	%end;
%end;

%else %if &m_purpose_cd eq LGD and %upcase(&CSB_UPDT_LGD_POST_BUILD_FLG) eq Y %then %do;
	/* Start: Modified for Hotfix 2014: S1109569*/

	%let m_abt_type_cd = MDL;

	%let m_level_cd = ;
	%let m_level_sk = ;
	proc sql noprint;
		select kupcase(lm.level_cd), lm.level_sk
				into :m_level_cd, :m_level_sk
		from &dabt_lib..modeling_abt_master mam
			inner join &dabt_lib..level_master lm
			on (mam.level_sk = lm.level_sk)
		where mam.abt_sk = &m_key_sk;
	quit;

    %let m_level_cd = &m_level_cd;
	%let m_level_sk = &m_level_sk;

	%let m_sub_of_analysis_cd = &m_level_cd;
	%let m_sub_rk_col_nm = ;

	/* Get key column name */
	/* Assumption: key constitutes of only one column */
	proc sql noprint;
		select source_column_nm into :m_sub_rk_col_nm
		from &dabt_lib..level_key_column_dtl lkcd inner join
			&dabt_lib..source_column_master scm
			on (lkcd.key_column_sk = scm.source_column_sk)
		where upcase(key_type_cd) = "RETAIN_KEY" and lkcd.level_sk = &m_level_sk; /*i18NOK:LINE */
	quit;
	%let m_sub_rk_col_nm = &m_sub_rk_col_nm;


	%let m_tbl_libref = &DABT_MODELING_ABT_LIBREF;
	%let m_tbl_nm = ;

	/* Get table name in which update is to be done */
	proc sql noprint;
		select abt_table_nm into :m_tbl_nm
		from &dabt_lib..modeling_abt_master
		where abt_sk = &m_key_sk;
	quit;
	%let m_tbl_nm = &m_tbl_nm;
	%let m_sub_retain_key_lib = &m_tbl_libref;
	%let m_sub_retain_key_tbl_nm = &m_tbl_nm;

	%let m_lgd_column_nm = LGD;
	%let m_ead_column_nm = EAD;
	%let m_def_dt_column_nm = INPUT_DEFAULT_DT;

	%csbmva_calc_upd_lgd_var_wrapper(m_abt_type_cd=&m_abt_type_cd, m_object_sk=&m_key_sk, m_subject_of_analysis_cd=&m_sub_of_analysis_cd,
								m_subject_retain_key_libref=&m_sub_retain_key_lib,m_subject_retain_key_tbl_nm=&m_sub_retain_key_tbl_nm,
								m_subject_rk_column_nm=&m_sub_rk_col_nm, m_to_be_upd_tbl_libref=&m_tbl_libref,
								m_to_be_upd_tbl_nm=&m_tbl_nm, m_to_be_upd_lgd_column_nm=&m_lgd_column_nm,
								m_to_be_upd_ead_column_nm=&m_ead_column_nm,m_to_be_upd_def_dt_column_nm=&m_def_dt_column_nm);
%end;
/* End: Modified for Hotfix 2014: S1109569*/

%mend csbmva_setdefaults;
