
	/* This macro is the wrapper code for preparing the actual ABT and load the ARM tables with actual result.
	   Parameters:	1.) m_scoring_template_id -  The scoring template id for which the process needs to be executed.
					2.) m_populate_arm_flg - The flag to decide whether the ARM needs to be populated or the user
											 will populate the ARM externally.
	*/
%macro dabt_build_act_abt_wrapper(m_scoring_template_id=,m_populate_arm_flg= Y) ;

	%let m_scoring_template_id = &m_scoring_template_id;
	%let m_project_sk = .;
	%let m_tmp_lib = ;
	%let m_src_lib = ;
	%let m_log_path_cab = ;
	%let m_log_file_nm = actual_calc_run_job_wrapper_&m_scoring_template_id..log;
	%let m_populate_arm_flg = %upcase(&m_populate_arm_flg);

	%if %symexist(job_rc)=0 %then %do;
		%global job_rc;
	%end;
	%if %symexist(sqlrc)=0 %then %do;
		%global sqlrc;
	%end;
	
	%let job_rc = 0;

	/*%let m_scoring_template_sk = .;*/
	%let m_scoring_template_sk = ;
	proc sql noprint;
		select scoring_template_sk into :m_scoring_template_sk
		from &lib_apdm..scoring_template_detail
		where scoring_template_id = "&m_scoring_template_id."; /*i18NOK:LINE*/
	quit;
	%let m_scoring_template_sk = &m_scoring_template_sk;

	%let m_scr_path_dabt = ;
	%let m_stg_path_dabt = ;
	%let m_stg_lib = ;
	%let m_query_file_path_dabt = ;
	%dabt_assign_libs(m_project_sk=&m_project_sk,tmp_lib=m_tmp_lib,
					src_lib=m_src_lib, log_path= m_log_path_cab,
					m_workspace_type=ACT, m_called_for_abt_processing_flg=Y,
					m_abt_or_template_sk=&m_scoring_template_sk., stg_lib=m_stg_lib, scr_path=m_scr_path_dabt,
					query_file_path=m_query_file_path_dabt, stg_path=m_stg_path_dabt);
	%dabt_err_chk(type=SQL);
	%dabt_err_chk(type=DATA);
	%dabt_err_chk(type=LIB);

	/* To clean any existing scratch files from last run of abt build if it was run with debug flag on*/
	%global m_abt_type;
	%let m_abt_type = ACT;
	%dabt_deassign_libs(m_abt_sk=&m_scoring_template_sk, m_abt_type=&m_abt_type, src_lib=&m_src_lib);
	%dabt_err_chk(type=SQL);
	%dabt_err_chk(type=DATA);
	%dabt_err_chk(type=LIB);

	%let m_log_path_cab = &m_log_path_cab;
	proc printto log = "&m_log_path_cab.%ktrim(&m_log_file_nm.)" &log_clear; /*i18NOK:LINE*/
	run;
	%dabt_err_chk(type=DATA);
	

	/* Get the model linked to the given scoring template */
	%let model_sk = ;
	proc sql noprint;
		select put(scoring_model_sk,12.) into :model_sk
		from &m_src_lib..scoring_model
		where scoring_template_sk = &m_scoring_template_sk.;
	quit;
	%dabt_err_chk(type=SQL);
	%let model_sk = &model_sk;
	
	%let m_level_sk = ;
	%let m_purpose_sk = .;
	proc sql noprint;
		select put(mm.level_sk,12.), put(pm.purpose_sk,12.) into :m_level_sk, :m_purpose_sk
		from &m_src_lib..scoring_model sm
			inner join &m_src_lib..model_master mm
			on (sm.model_sk = mm.model_sk)
			inner join &m_src_lib..project_master pm
			on (mm.project_sk = pm.project_sk)
		where sm.scoring_model_sk = &model_sk.;
	quit;
	%dabt_err_chk(type=SQL);
	%let m_level_sk = &m_level_sk;
	%let m_purpose_sk = &m_purpose_sk;
	
	%let tmp_act_bld_dttm = ;
	/* This macro returns the latest build date depending on the input mart load date */
	/* tmp_act_bld_dttm is more like latest date for which input/foundation/source mart is successfully loaded*/
	%dabt_get_latest_bld_date_info(m_scoring_template_sk=&m_scoring_template_sk, m_ltst_bld_dt=tmp_act_bld_dttm);
	
	/*Get the scoring_control_detail_sk from APDM scoring_control_detail table for all the templates corresponding to
	  the scoring_template_id, for which Scoring processs is successfull but the actual run is pending.*/
	%let m_control_detail_sk_lst = ;
	proc sql noprint;
		select put(scoring_control_detail_sk,12.) into :m_control_detail_sk_lst separated by ','
		from &m_src_lib..scoring_control_detail
		where scoring_template_sk = &m_scoring_template_sk. 
			and kupcase(actual_result_run_active_flg) = "Y"
			and load_arm_status_sk = (select status_sk from &m_src_lib..status_master
							where kupcase(strip(status_cd)) = "SUCCESS");
	quit;
	%dabt_err_chk(type=SQL);
	
	%if  &job_rc le 4 %then %do;
		%if "&m_control_detail_sk_lst." ne "" %then %do; /*i18NOK:LINE*/
			%let m_control_detail_cnt = %eval(%sysfunc(countc(%quote(&m_control_detail_sk_lst),","))+1);
		
			%do m_cntrl = 1 %to &m_control_detail_cnt;
				%let m_cntrl_sk_tkn = %scan(%quote(&m_control_detail_sk_lst),&m_cntrl,%str(,));

				%let m_act_bld_dttm = ;
				%let m_act_run_flg = ;
				/* This macro returns the flag whether the actual value abt is to be build, and if Y then returns the
					build date */
				%dabt_get_act_abt_bld_info(m_scoring_control_detail_sk=&m_cntrl_sk_tkn.,
								m_ltst_mart_load_dttm=&tmp_act_bld_dttm,
								act_run_flg=m_act_run_flg,					
								act_bld_dttm=m_act_bld_dttm);

				%if &job_rc le 4 %then %do;
					%if &m_act_run_flg eq Y %then %do;

						%let m_scr_bld_dttm = .;
						data _null_;
							set &m_src_lib..scoring_control_detail;
							where scoring_control_detail_sk = &m_cntrl_sk_tkn.;
							call symput('m_scr_bld_dttm',scoring_as_of_dttm); /*i18NOK:LINE*/
						run;
						%dabt_err_chk(type=DATA);
						%let m_scr_bld_dttm = &m_scr_bld_dttm;

						/*ACT_<SCORING_TEMPLATE_ID>_<SCORING_AS_OF_DATE(DDMMYY)>_<ACTUAL_AS_OF_DATE(DDMMYY)>*/
						%let m_scr_dttm = ;
						%let m_act_dttm = ;
						data _null_;
							scr_date = put(datepart(&m_scr_bld_dttm.), date9.); /*i18NOK:LINE*/
							act_date = put(datepart(&m_act_bld_dttm.), date9.); /*i18NOK:LINE*/
							call symput('m_scr_dttm',scr_date); /*i18NOK:LINE*/
							call symput('m_act_dttm',act_date); /*i18NOK:LINE*/
						run;
						%let m_scr_dttm = &m_scr_dttm.;
						%let m_act_dttm = &m_act_dttm;

						%let m_actual_calc_abt_nm = ACT_&m_scoring_template_id._&m_scr_dttm._&m_act_dttm;
						%let m_actual_calc_abt_nm = &m_actual_calc_abt_nm.;

						%if &job_rc le 4 %then %do;
							/* if the build status is successful then the macro m_abt_sts will have value 0,
								else 1, this is returned by the  dabt_build_act_abt macro */
							%let m_abt_sts = ;
							%dabt_build_act_abt(m_scoring_control_detail_sk=&m_cntrl_sk_tkn.,m_scoring_template_sk=&m_scoring_template_sk,
											m_scoring_dttm=&m_scr_bld_dttm, m_actual_bld_dttm=&m_act_bld_dttm,
											m_actual_calc_abt_nm=&m_actual_calc_abt_nm, m_abt_bld_status=m_abt_sts, m_log_file_nm=&m_log_file_nm.);
							
							/*Start: Modified as part of hotfix 2014: S1109569 */
							%let m_act_result_dtl_sk = ;
							proc sql noprint;
								select actual_result_control_dtl_sk into :m_act_result_dtl_sk
								from &m_src_lib..actual_result_control_detail
								where scoring_control_detail_sk = &m_cntrl_sk_tkn.
									and actual_result_as_of_dttm = &m_act_bld_dttm.;
							quit;
							%let m_act_result_dtl_sk = &m_act_result_dtl_sk;
							%dabt_err_chk(type=SQL);
							
							%csbmva_post_actual_calc_wrapper(m_action_cd=ACT_JOB, m_purpose_sk=&m_purpose_sk, m_key_sk=&m_act_result_dtl_sk);
							%dabt_err_chk(type=SQL);
							%dabt_err_chk(type=DATA);
							%dabt_err_chk(type=LIB);
							/*End: Modified as part of hotfix 2014: S1109569 */
						%end;
						%else %do;
							%let m_abt_sts = 1;
						%end;

						proc printto log = "&m_log_path_cab.%ktrim(&m_log_file_nm.)" ; /*i18NOK:LINE*/
						run;
						%dabt_err_chk(type=DATA);
						
						
						%if &m_abt_sts eq 0 and &m_populate_arm_flg = Y %then %do;
							%let m_act_abt_libref = ;
							%dabt_assign_abt_library(m_abt_sk=&m_scoring_template_sk, m_abt_lib_ref=m_act_abt_libref, m_abt_type=ACT);
							%dabt_err_chk(type=SQL);
							%dabt_err_chk(type=DATA);
							%dabt_err_chk(type=LIB);

							%if &job_rc le 4 %then %do;
								/* Populate the analytical reault area with the scoring results */
								%dabt_populate_actual_result(m_scoring_template_id = &m_scoring_template_id., 
																m_input_table_nm = &m_act_abt_libref..&m_actual_calc_abt_nm., 
																m_scoring_bld_dttm= &m_scr_bld_dttm.,
																m_actual_bld_dttm= &m_act_bld_dttm.);

							%end;
						%end;

						%if &job_rc le 4 %then %do;
							/* This macro variable will be assigned the value N, if the post action macro executes without error,
							 else will have value Y. */
							%let post_action_err_flg = ;
							/*Start: Modified as part of hotfix 2014: S1109569 */
							%dabt_call_post_action_macro(m_action_cd=ACT_JOB, m_purpose_sk=&m_purpose_sk,
								m_actual_result_dtl_sk=&m_act_result_dtl_sk, m_post_action_flg=post_action_err_flg);
							/*%dabt_call_post_action_macro(m_action_cd=ACT_JOB, m_purpose_sk=&m_purpose_sk,
															m_scoring_template_sk=&m_scoring_template_sk, m_post_action_flg=post_action_err_flg);*/
							/*End: Modified as part of hotfix 2014: S1109569 */
							%dabt_err_chk(type=DATA);
							%dabt_err_chk(type=SQL);
							%dabt_err_chk(type=LIB);
							%if &post_action_err_flg = Y %then %do;
								%let job_rc = 5;
							%end;
						%end;

					%end;
				%end;
			%end;
		%end;/*"&m_control_detail_sk_lst." ne "" */
	%end;


	proc printto log=log;
	run;

%mend dabt_build_act_abt_wrapper;
