
	/* This macro builds the back testing abt and returns the status in the macro 
		m_abt_bld_status as 0 for success and 1 for failure */
%macro dabt_build_bck_plg_abt_wrapper(m_model_id=, m_pool_scheme_no=, m_mdl_type=, m_abt_bld_dt=,m_abt_stk_flg=, m_abt_stk_size=, m_replace_abt_flg=, 
							m_perform_scoring_flg =,m_abt_libref=,	m_abt_tbl_nm=, m_scored_ds_nm=, 
							m_scored_libref=, m_abt_lst_prc_by_usr=, m_abt_build_status=,
							m_unique_id_for_scr_tbl=, m_ref_path_bld=, m_log_file_nm=);

	%let m_model_id = &m_model_id;
	%let m_pool_scheme_no = &m_pool_scheme_no;
	%let m_abt_libref = &m_abt_libref;
	%let m_abt_tbl_nm = &m_abt_tbl_nm;
	%let m_mdl_type = &m_mdl_type;
	%let m_mdl_type_low = %lowcase(&m_mdl_type);
	%let m_perform_scoring_flg = %upcase(&m_perform_scoring_flg);
	%let m_replace_abt_flg = %upcase(&m_replace_abt_flg);

	%let clear_log_flg = N;
	%if &m_log_file_nm eq %then %do;
		%let m_log_file_nm = build_&m_mdl_type_low._abt_&m_model_sk..log;;
		%let clear_log_flg = Y;
	%end;
	
	%if %symexist(job_rc)=0 %then %do;
		%global job_rc;
	%end;
	%if %symexist(sqlrc)=0 %then %do;
		%global sqlrc;
	%end;
	
	%let job_rc = 0;
	
	%let m_src_lib = &lib_apdm.;
						
	%let m_project_sk = ;
	%let m_model_sk = ;
	%let m_level_sk = ;
	proc sql noprint;
		select project_sk, model_sk, level_sk into :m_project_sk, :m_model_sk, :m_level_sk
		from &m_src_lib..model_master
		where model_id = "&m_model_id."; /*i18NOK:LINE*/ 
	quit;
	%dabt_err_chk(type=SQL);
	%let m_project_sk = &m_project_sk;
	%let m_model_sk = &m_model_sk;
	%let m_level_sk = &m_level_sk;
	
	%let m_tmp_lib = ;
	%let m_src_lib = ;
	%let m_log_path = ;
	%let m_scr_path = ;
	%let m_stg_path = ;
	%let m_stg_lib = ;
	%let m_query_file_path = ;
	%let m_deployed_code_path = ;
	/* assigns the temporary library and ADM library */
	%dabt_assign_libs(m_project_sk=,tmp_lib=m_tmp_lib,
					 src_lib=m_src_lib, log_path= m_log_path,
					 m_workspace_type=&m_mdl_type, m_called_for_abt_processing_flg=Y,
					 m_abt_or_template_sk=&m_model_sk., stg_lib=m_stg_lib, scr_path=m_scr_path,
					query_file_path=m_query_file_path, stg_path=m_stg_path, m_abt_tbl_nm=&m_abt_tbl_nm, 
					deployed_code_path=m_deployed_code_path, m_related_path=%quote(&m_ref_path_bld));
	%dabt_err_chk(type=SQL);
	%dabt_err_chk(type=DATA);
	%dabt_err_chk(type=LIB);

	/* To clean any existing scratch files from last run of abt build if it was run with debug flag on*/
	%dabt_deassign_libs(m_abt_sk=&m_model_sk, m_abt_type=&m_mdl_type, src_lib=&m_src_lib, called_at_start_of_process_flg=Y,
							 m_abt_tbl_nm=&m_abt_tbl_nm, m_related_path=%quote(&m_ref_path_bld));
	%dabt_err_chk(type=SQL);
	%dabt_err_chk(type=DATA);
	%dabt_err_chk(type=LIB);
		
	/* If the wrapper code didn't pass the log file name, then the earlier content of the file is removed. */
	%if &clear_log_flg eq Y %then %do;		
		proc printto log = "&m_log_path.%ktrim(&m_log_file_nm.)" &dabt_log_clear; /*i18NOK:LINE*/
		run;
		%dabt_err_chk(type=DATA);
		
	%end;
	%else %do;
		proc printto log = "&m_log_path.%ktrim(&m_log_file_nm.)"; /*i18NOK:LINE*/
		run;
		%dabt_err_chk(type=DATA);
	%end;				 
		
	%if &m_mdl_type = BCK %then %do;
		%let m_scr_libref = &DABT_BACKTESTING_SCR_LIBREF.;
	%end;
	%else %do;
		%let m_scr_libref = &DABT_POOLING_SCR_LIBREF.;
	%end;
	%if &m_abt_libref eq %then %do;
		%if &m_mdl_type = BCK %then %do;
			%let m_abt_libref = &DABT_BACKTESTING_ABT_LIBREF.;
		%end;
		%else %do;
			%let m_abt_libref = &DABT_POOLING_ABT_LIBREF.;
		%end;
	%end;

	%if &m_abt_tbl_nm eq %then %do;
		%put %sysfunc(sasmsg(SASHELP.dabt_error_messages,ERR_BLD_BCK_ABT_NM_NOT_ASSIGNED,N,m_abt_tbl_nm));
		%return;
	%end;
	%if &m_perform_scoring_flg eq Y %then %do;
		%if &m_scored_ds_nm eq %then %do;
			%put %sysfunc(sasmsg(SASHELP.dabt_error_messages,ERR_BLD_BCK_SCRD_ABT_NM_NOT_ASSIGNED,N,m_scored_ds_nm));
			%return;
		%end;
		%if &m_scored_libref eq %then %do;
			%let m_scored_libref = &m_abt_libref;
		%end;
	%end;

	%if %sysfunc(exist(&m_abt_libref..&m_abt_tbl_nm)) %then %do;
		%if &m_replace_abt_flg eq Y %then %do;
			proc sql noprint;
				drop table &m_abt_libref..&m_abt_tbl_nm;
			quit;
			%dabt_err_chk(type=SQL);
		%end;
		%else %do;
			%put %sysfunc(sasmsg(SASHELP.dabt_error_messages,ERR_BLD_BCK_ABT_TBL_EXSTS,N,&m_abt_tbl_nm,&m_abt_libref.));
			%return;
		%end;
	%end;
	%if &m_perform_scoring_flg eq Y %then %do;
		%if %sysfunc(exist(&m_scored_libref..&m_scored_ds_nm)) %then %do;
			%if &m_replace_abt_flg eq Y %then %do;
				proc sql noprint;
					drop table &m_scored_libref..&m_scored_ds_nm;
				quit;
				%dabt_err_chk(type=SQL);
			%end;
			%else %do;
				%put %sysfunc(sasmsg(SASHELP.dabt_error_messages,ERR_BLD_BCK_ABT_TBL_EXSTS,N,&m_scored_ds_nm,&m_scored_libref));
				%return;
			%end;
		%end;
	%end;

	%if &m_mdl_type = BCK %then %do;
		/* The UI for backtesting doesn't update this table, hence it is updated here */
		%let m_lst_prc_dt = %sysfunc(datetime());
		proc sql noprint ;
			update &m_src_lib..model_master
			set last_processed_dttm = &m_lst_prc_dt,
				last_processed_by_user = "&m_abt_lst_prc_by_usr", /*i18NOK:LINE*/
				last_evaluation_run_dttm = &m_lst_prc_dt,
	/*			last_evaluation_stacked_type_flg = "&m_abt_stk_flg",*/
				last_evaluation_status_sk = (select status_sk from &m_src_lib..status_master
								where kupcase(strip(status_cd)) = "RUNNING")
			where model_sk = &m_model_sk.;
		quit;
		%dabt_err_chk(type=SQL);
	%end;

	%let m_abt_sts = 0;
	%if &job_rc le 4 %then %do;
		%dabt_build_abt(m_project_sk=&m_project_sk, m_model_sk=&m_model_sk, m_abt_bld_dt=&m_abt_bld_dt, m_abt_stk_flg=&m_abt_stk_flg, m_abt_stk_size=&m_abt_stk_size,
										m_abt_type=&m_mdl_type, m_abt_libref=&m_abt_libref, m_scr_libref=&m_scr_libref,
										m_abt_tbl_nm=&m_abt_tbl_nm, m_abt_bld_sts=m_abt_sts,
										m_unique_id_for_tmp_tbl=&m_unique_id_for_scr_tbl, 
										m_ref_path_bld=%quote(&m_ref_path_bld), m_log_file_nm=&m_log_file_nm);
										
		proc printto log = "&m_log_path.%ktrim(&m_log_file_nm.)"; /*i18NOK:LINE*/
		run;
		%dabt_err_chk(type=DATA);
	%end;

	%let app_scr_sts = 0;
	%if &job_rc le 4 and &m_abt_sts eq 0 %then %do;
		%if &m_perform_scoring_flg eq Y %then %do;
			%let m_scr_path = ;
			%let m_tmp_lib = ;
			%let m_src_lib = ;
			%let m_scoring_code_path = ;
			/* assigns the temporary library and ADM library */
			%dabt_assign_libs(m_project_sk=, src_lib=m_src_lib,tmp_lib=m_tmp_lib,m_workspace_type=&m_mdl_type, m_called_for_abt_processing_flg=N,
							 m_abt_or_template_sk=&m_model_sk., scr_path=m_scr_path, scoring_code_path=m_scoring_code_path, m_related_path=%quote(&m_ref_path_bld));
			%dabt_err_chk(type=SQL);
			%dabt_err_chk(type=DATA);
			%dabt_err_chk(type=LIB);

			%let app_scr_sts = ;
			%dabt_apply_scoring_code(m_scoring_abt_table_nm=&m_abt_libref..&m_abt_tbl_nm,
									m_output_table_nm=&m_scored_libref..&m_scored_ds_nm,
									m_level_sk=&m_level_sk,m_port_path=&m_scoring_code_path, m_scr_status=app_scr_sts,
									m_scoring_code_file_nm=scoring_code_&m_model_id..sas);
									
			proc printto log = "&m_log_path.%ktrim(&m_log_file_nm.)"; /*i18NOK:LINE*/
			run;
			%dabt_err_chk(type=DATA);
		%end;
	%end;

	%if &job_rc le 4 and &m_abt_sts eq 0 and &app_scr_sts eq 0 %then %do;
		/* return the status code 0 for success */
		%let &m_abt_build_status = 0;
	%end;
	%else %do;
		/* return the status code 1 for failure */
		%let &m_abt_build_status = 1;
	%end;

	%if &&&m_abt_build_status eq 0 %then %do;
		%let m_purpose_sk = .;
		proc sql noprint;
			select purpose_sk into :m_purpose_sk
			from &m_src_lib..project_master
			where project_sk = &m_project_sk.;
		quit;
		%dabt_err_chk(type=SQL);
		%let m_purpose_sk = &m_purpose_sk.;

		/* This macro variable will be assigned the value N, if the post action macro executes without error,
		 else will have value Y. */
		%let post_action_err_flg = ;
		/* Modified for pooling added as part of hotfix 2014: S1109569 */
		%if &m_mdl_type = BCK %then %do;
			%dabt_call_post_action_macro(m_action_cd=BCK_JOB, m_purpose_sk=&m_purpose_sk,
											m_model_sk=&m_model_sk, m_post_action_flg=post_action_err_flg);
		%end;
		%else %do;
			%dabt_call_post_action_macro(m_action_cd=PLG_JOB, m_purpose_sk=&m_purpose_sk,
								m_model_sk=&m_model_sk, m_pool_scheme_no=&m_pool_scheme_no, m_post_action_flg=post_action_err_flg);
		%end;
		%dabt_err_chk(type=DATA);
		%dabt_err_chk(type=SQL);
		%dabt_err_chk(type=LIB);
		%if &post_action_err_flg = Y %then %do;
			%let job_rc = 5;
		%end;
		
		%if &job_rc le 4 %then %do;
			/* return the status code 0 for success */
			%let &m_abt_build_status = 0;
		%end;
		%else %do;
			/* return the status code 1 for failure */
			%let &m_abt_build_status = 1;
		%end;
	%end;
	
	%if &m_mdl_type = BCK %then %do;
		%let m_lst_prc_dt = %sysfunc(datetime());
		proc sql noprint ;
			update &m_src_lib..model_master
			set last_processed_dttm = &m_lst_prc_dt,
				last_evaluation_status_sk = (select status_sk from &m_src_lib..status_master
										where kupcase(strip(status_cd)) = 
									%if &&&m_abt_build_status eq 0 %then %do;
										"SUCCESS"
									%end;
									%else %do;
										"FAIL"
									%end;
										)
			where model_sk = &m_model_sk.;
		quit;
		%dabt_err_chk(type=SQL);
	%end;
	
	proc printto log=log;
	run;

%mend dabt_build_bck_plg_abt_wrapper;
